#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#define F_CPU 16000000

void USART0_init()
{
	/*Setez baund rate la 38400.*/
	UBRR0H = 0;
	UBRR0L = 103;

	/*Pornesc transmitatorul.*/
	UCSR0B = (1<<RXEN0);
	UCSR0B |= (1<<RXCIE0);

}

char USART0_receive()
{
	while (!(UCSR0A & (1<<RXC0)));

	return UDR0;
}

/*Setez porturile corespunzatoare led-urilor ca iesiri.*/
void ledInit()
{
	DDRA |= (1<<PA6);
	DDRA |= (1<<PA7);

	PORTA &= ~(1<<PA6);
	PORTA &= ~(1<<PA7);	
}

/*Aprind led-urile din fata.*/
void frontLightsOn()
{
	PORTA |= (1<<PA7);
}

/*Sting led-urile din fata.*/
void frontLightsOff()
{
	PORTA &= ~(1<<PA7);
}

/*Aprind led-urile din spate.*/
void backLightsOn()
{
	PORTA |= (1<<PA6);
}

/*Sting led-urile din spate.*/
void backLightsOff()
{
	PORTA &= ~(1<<PA6);
}

/*Setez portul buzzer-ului ca iesire*/
void hornInit()
{
	DDRC |= (1<<PC6);
	PORTC &= ~(1<<PC6);
}

/*Pornesc claxonul.*/
void hornOn()
{
	PORTC |= (1<<PC6);
}

/*Opresc claxonul.*/
void hornOff()
{
	PORTC &= ~(1<<PC6);
}

/*Setez pinii corespunzatori motoarelor.*/
void initMotor()
{
	DDRB |= (1<<PB1);
	DDRB |= (1<<PB2);
	DDRB |= (1<<PB3);
	DDRB |= (1<<PB4);
	DDRD |= (1<<PD4);
	DDRD |= (1<<PD5);

	PORTB &= ~(1<<PB1);
	PORTB &= ~(1<<PB2);
	PORTB &= ~(1<<PB3);
	PORTB &= ~(1<<PB4);
	PORTD |= (1<<PD4);
	PORTD |= (1<<PD5);

}

void moveForward()
{
	PORTB &= ~(1<<PB1);
	PORTB |= (1<<PB2);

}

void moveBack()
{
	PORTB &= ~(1<<PB2);
	PORTB |= (1<<PB1);
}

void moveLeft()
{
	PORTB &= ~(1<<PB4);
	PORTB |= (1<<PB3);
}

void moveRight()
{
	PORTB &= ~(1<<PB3);
	PORTB |= (1<<PB4);
}


void stopMotor()
{
	PORTB &= ~(1<<PB1);
	PORTB &= ~(1<<PB2);
	PORTB &= ~(1<<PB3);
	PORTB &= ~(1<<PB4);
}

int main()
{
	USART0_init();
	ledInit();
	hornInit();
	initMotor();

	char command;

	while (1) {
		command = USART0_receive();
		switch(command) {
			case 'W':
				frontLightsOn();
				continue;
			case 'w':
				frontLightsOff();
				continue;
			case 'U':
				backLightsOn();
				continue;
			case 'u':
				backLightsOff();
				break;
			case 'V':
				hornOn();
				break;
			case 'v':
				hornOff();
				break;
			case 'F':
				moveForward();
				break;
			case 'B':
				moveBack();
				break;
			case 'L':
				moveLeft();
				break;
			case 'R':
				moveRight();
				break;
			case 'G':
				moveForward();
				moveLeft();
				break;
			case 'I':
				moveForward();
				moveRight();
				break;
			case 'H':
				moveBack();
				moveLeft();
				break;
			case 'J':
				moveBack();
				moveRight();
				break;
			default:
				stopMotor();
				break;
		}
		
	}
}